import { defineStore } from 'pinia'
import { useUserStore } from './user'
import { apiGetUserRoute, type UserRoute } from '@/api/common/login'
import type { MenuPermissionInfo } from '@/api/sys/model/menuModel'
import type { AppRouteRecordRaw } from '@/router/types'
import Layout from '@/layouts/default/index.vue'
import { basicRoutes } from '@/router/routes/index'

interface PermissionState {
  backMenuList: UserRoute[]
  btnPermissionList: UserRoute[]
  menuList: AppRouteRecordRaw[]
  isDynamicAddedRoute: boolean
}
const modules = import.meta.glob('../../views/**/*.vue')

const filterAsyncRoute = (routes: UserRoute[]): AppRouteRecordRaw[] => {
  if (!routes) return []

  const res: AppRouteRecordRaw[] = []

  routes.forEach((_route: UserRoute) => {
    const route: AppRouteRecordRaw = {
      path: _route.permissionUrl,
      component: _route.permissionType === '0' ? Layout : loadView(_route.permissionComponent),
      meta: {
        hideMenu: !!_route.hidden,
        isLink: _route.internalOrExternal,
        keepAlive: _route.keepAlive,
        title: _route.permissionName,
        icon: _route.permissionIcon || ''
      },
      children: _route.childNode !== null ? filterAsyncRoute(_route.childNode) : []
    }
    res.push(route)
  })

  return res
}

const loadView = (path: string, suffix = '.vue', prefix = '../../views') => {
  const link = modules[prefix + path + suffix]

  if (link === undefined) {
    console.error('Failed to resolve view: ' + path)
  }
  return link
}

const getMenu = (routes: AppRouteRecordRaw[]) => {
  const rt = [...basicRoutes, ...routes]

  // 过滤隐藏的目录
  const menu = rt.filter((_r) => !!!_r.meta?.hideMenu)
  return menu
}

export const usePermissionStore = defineStore('app-permission', {
  state: (): PermissionState => {
    return {
      btnPermissionList: [],
      backMenuList: [],
      menuList: [],
      isDynamicAddedRoute: false
    }
  },
  getters: {
    getIsDynamicAddedRoute(): boolean {
      return this.isDynamicAddedRoute
    },
    getBackMenuList(): UserRoute[] {
      return this.backMenuList
    },
    getMenuList(): AppRouteRecordRaw[] {
      return this.menuList
    }
  },
  actions: {
    setIsDynamicAddedRoute(added: boolean) {
      this.isDynamicAddedRoute = added
    },
    setBtnPermissionListAction(list: UserRoute[]) {
      this.btnPermissionList = list
    },
    setMenuListAction(list: AppRouteRecordRaw[]) {
      this.menuList = list
    },
    // 设置返回的菜单列表
    setBackMenuListAction(list: UserRoute[]) {
      this.backMenuList = list
    },
    // 获取用户权限菜单
    async getRouteAction() {
      try {
        const { result } = await apiGetUserRoute()

        this.setBackMenuListAction(result.permissionSet)
        this.setBtnPermissionListAction(result.btnPermissionSet)
        return result.permissionSet
      } catch (error) {
        return Promise.reject(error)
      }
    },
    // 生成路由
    async buildRoutesAction() {
      const broute = await this.getRouteAction()

      const routes: AppRouteRecordRaw[] = filterAsyncRoute(broute)

      const menuList = getMenu(routes)

      this.setMenuListAction(menuList)
      return routes
    }
  },
  persist: {
    paths: ['menuList', 'btnPermissionList']
  }
})
