import type { OrgDto, OrgModel } from '@/api/sys/model/orgModel'
import {
  apiDeleteOrg,
  apiGetOrgList,
  apiGetOrgTree,
  apiInsertOrg,
  apiUpdateOrg,
  type OrgTreeModel
} from '@/api/sys/org'
import { Modal, message } from 'ant-design-vue'
import type { Rule } from 'ant-design-vue/es/form'
import { ExclamationCircleOutlined } from '@ant-design/icons-vue'

import { computed, createVNode, onMounted, reactive, ref, unref, watch } from 'vue'

const initState = (): OrgDto => {
  return {
    orgId: '',
    orgCategory: '1',
    orgCode: '',
    orgDes: '',
    orgName: '',
    pid: '',
    address: ''
  }
}
const rules: Record<string, Rule[]> = {
  orgName: [{ required: true, message: '请输入组织名称', trigger: 'change' }],
  orgCode: [{ required: true, message: '请输入组织编码', trigger: 'change' }]
}

export function useOrgTable() {
  const tableData = ref<OrgModel[]>([])
  const tableRef = ref()
  const loading = ref(false)

  const queryData = async () => {
    const { result } = await apiGetOrgList()
    tableData.value = result
  }

  return {
    tableData,
    loading,
    tableRef,
    queryData
  }
}

export function useOrgForm() {
  const open = ref(false)
  const orgFormRef = ref()
  const orgFormState = reactive(initState())
  const orgFormRules = reactive<Record<string, Rule[]>>(rules)
  const treeData = ref<OrgTreeModel[]>([])
  let currentOrg = reactive<OrgModel | {}>({})
  const btnLoadding = ref(false)
  const { tableData, tableRef } = useOrgTable()


  const isModify = computed(() => orgFormState.orgId !== '')
  const validate = () => orgFormRef.value.validate()
  const cleanValidate = () => orgFormRef.value.clearValidate()
  const resetFrom = () => Object.assign(unref(orgFormState), initState())
  const closeModal = () => {
    resetFrom()
    cleanValidate()
    open.value = false
  }



  const getTreeData = async () => {
    const { result } = await apiGetOrgTree()
    treeData.value = result
  }

  const modifyOrg = (row: OrgModel) => {
    currentOrg = unref(row)
    Object.assign(unref(orgFormState), unref(row))
    open.value = true
  }

  const deleteOrg = async (row: OrgModel) => {
    const tref = unref(tableRef)
    const td = unref(tableData)
    Modal.confirm({
      title: '温馨提醒',
        icon: createVNode(ExclamationCircleOutlined),
        content: `是否确认删除${row.orgName}?`,
        okText: '确认',
        cancelText: '取消',
        onOk: async () => {
          try {
            await apiDeleteOrg({ids: row.orgId})
            message.success('删除成功！')
            td.splice(tref.getRowIndex(row), 1)
          } catch {}
        }
    })

  }

  const onFinish = async () => {
    const td = unref(tableData)
    btnLoadding.value = true
    try {
      await validate()
      const { result } = isModify.value
        ? await apiUpdateOrg(orgFormState)
        : await apiInsertOrg(orgFormState)

      if (isModify.value) {
        Object.assign(unref(currentOrg), result)
      } else {
        td.push(result)
      }
      message.success('保存成功！')
      setTimeout(() => {
        closeModal()
      }, 500)
    } catch {}
    btnLoadding.value = false
  }

  watch(
    () => open.value,
    () => {
      if (open.value) {
        getTreeData()
      }
    },
    { immediate: true }
  )

  return {
    open,
    orgFormRef,
    btnLoadding,
    orgFormState,
    orgFormRules,
    treeData,
    isModify,
    onFinish,
    deleteOrg,
    closeModal,
    modifyOrg
  }
}
